# coding: utf-8
import logging
import os
from concurrent import futures

import grpc

from taichu_serve.grpc_predict_v2_pb2_grpc import add_GRPCInferenceServiceServicer_to_server
from taichu_serve.app import app, init_model_service_instance
from taichu_serve.grpc_server import GrpcModelService, GrpcServerInterceptor
from taichu_serve.settings import parse_args

LOGGER = logging.getLogger(__name__)


def init_project(name="project"):
    # 在当前目录下创建项目目录
    if not os.path.exists(name):
        os.mkdir(name)

    # 获取当前目录
    current_path = os.path.abspath(os.path.dirname(__file__))
    # 获取模板目录
    template_path = os.path.join(current_path, "template")
    # 拷贝模板目录下的文件到项目目录
    for file in os.listdir(template_path):
        if file == "__pycache__":
            continue
        if file.endswith("__init__.py"):
            continue

        if os.path.isdir(os.path.join(template_path, file)):
            os.mkdir(os.path.join(name, file))
            for sub_file in os.listdir(os.path.join(template_path, file)):
                if sub_file == "__pycache__":
                    continue
                if sub_file.endswith("__init__.py"):
                    continue
                with open(os.path.join(template_path, file, sub_file), "r") as f:
                    content = f.read()
                print("create file: ", os.path.join(name, file, sub_file))
                with open(os.path.join(name, file, sub_file), "w") as f:
                    f.write(content)
            continue

        with open(os.path.join(template_path, file), "r") as f:
            content = f.read()
        print("create file: ", os.path.join(name, file))
        with open(os.path.join(name, file), "w") as f:
            f.write(content)
    print("init project done!")


def cli():
    args = parse_args()
    if args.action == "init":
        init_project()
        return

    init_model_service_instance()
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=args.max_concurrent_requests),
                         maximum_concurrent_rpcs=args.max_concurrent_requests + 5,
                         interceptors=[GrpcServerInterceptor()])

    add_GRPCInferenceServiceServicer_to_server(GrpcModelService(), server)
    server.add_insecure_port(f'[::]:{args.grpc_port}')

    server.start()
    LOGGER.info("grpc server start at port %s", args.grpc_port)

    if args.grpc_only:
        server.wait_for_termination()

    app.run("0.0.0.0", args.http_port)
    return app
